
//
//  drawFunction.cpp
//  collide
//
//  Created by Andrew Lee on 11/30/15.
//
//

#include "drawFunction.h"
#include "instructions.h"
#include <stdlib.h>     /* srand, rand */


drawFunction::drawFunction(testApp* ptr)
{
    myApp = ptr;
}

void drawFunction::draw(){

//    myApp->video.draw(0, 0);
//    myApp->video.play();
    
    // Drawing Instructions
    // ---------------------------
    instructions instructions(myApp);
    instructions.drawInstructions();
    
    
    int PORTAL_RADIUS = myApp->PORTAL_RADIUS;
    
    myApp->myCamera.begin();
    
    // DRAW TRAJECTORIES
    // ----------------------------
    if (myApp->showBallTrajectories){
        for (int i = 0; i < myApp->ballTrajectories.size(); i++){
            vector<b2Vec2> trajectoryPath = myApp->ballTrajectories[i];
            ofSetColor(1, 0, 0);
            glBegin(GL_LINES);
            for (int j = 0; j < trajectoryPath.size(); j++){
                glVertex2f(trajectoryPath[j].x, trajectoryPath[j].y);
            }
            glEnd();
        }
    }

    
    // DRAW CIRCLES
    // -----------------------
    for (int i = 0; i < myApp->circles.size(); i++){
        myApp->checkTeleport(i); // check for vortex intersection
    }
    
    // DRAWING CIRCLE
    for(size_t i=0; i<myApp->circles.size(); i++) {
        int numCircles = i;
        ofFill();
        SoundData * data = (SoundData*)myApp->circles[numCircles].get()->getData();
        if(data && data->bHit) ofSetHexColor(0xff0000);
        else ofSetHexColor(0x4ccae9);
        myApp->circles[numCircles].get()->draw();
        
    }
    
    // DRAW RECTANGLES
    // -------------------------------
    ofSetHexColor(0x4ccae9);
    for (int i = 0; i < myApp->rectangles.size(); i++){
        if (myApp->rectangles[i].ptr->body->isHit()){
            ofSetHexColor(0xff0000);
        } else {
            ofSetHexColor(0x4ccae9);
        }
        shared_ptr<ofxBox2dRect> rec = myApp->rectangles[i].ptr;
        rec->draw();
        
        // draw notes on rectangles
        string text = instructions.pianoNotes[rec->body->getSoundFile()];
        ofSetHexColor(0x000000);
        ofDrawBitmapString(text, rec->getPosition().x, rec->getPosition().y);
    }
    
    // DRAWS CURRENTLY DRAGGED RECTANGLE/TRAMPOLINE
    if ((myApp->currentMode == RECTANGLE || myApp->currentMode == TRAMPOLINE) && myApp->currentlyDrawingRectangle){
        if (myApp->currentMode == RECTANGLE){
            ofSetHexColor(0x4ccae9); // blue
        } else {
            ofSetHexColor(0xffff00); //yellow
        }
        ofPtr<ofxBox2dRect> rect = ofPtr<ofxBox2dRect>(new ofxBox2dRect);
        rect.get()->setPhysics(0,0.2,0.2);
        ofVec2f pos = ofVec2f(myApp->currentRectangle.x, myApp->currentRectangle.y);
        float w = myApp->currentRectangle.w;
        float h = myApp->currentRectangle.h;
        rect.get()->setup(myApp->box2d.getWorld(), pos.x, pos.y, w, h);
        rect.get()->body->SetType(b2_staticBody);
        b2Body* rectBody = rect.get()->body;
        rectBody->SetTransform(rectBody->GetPosition(), myApp->currentRectangle.angle);
        rectBody->setSoundFile(myApp->currentSound);
        rect->draw();
        rect->destroy();
    }
    
    // DRAW currently dragged stream/portals
    if (myApp->currentMode == SELECTION && myApp->selected.dragged){
        ofNoFill();
        if (myApp->selected.mode == STREAM){
            ofSetColor(0);
            ofDrawCircle(myApp->selected.intermittent.x, myApp->selected.intermittent.y, myApp->CIRCLE_RADIUS);
        } else if (myApp->selected.mode == PORTAL_IN){
            ofSetColor(255);
            float x = myApp->selected.intermittent.x;
            float y = myApp->selected.intermittent.y;
            myApp->inPortal.draw(x - PORTAL_RADIUS, y - PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
            int num = myApp->selected.metaInt;
            ofSetHexColor(0x000000);
            ofDrawBitmapString(num, x + PORTAL_RADIUS / 2, y + PORTAL_RADIUS);
        } else if (myApp->selected.mode == PORTAL_OUT){
            ofSetColor(255);
            float x = myApp->selected.intermittent.x;
            float y = myApp->selected.intermittent.y;
            myApp->outPortal.draw(x - PORTAL_RADIUS, y - PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
            int num = myApp->selected.metaInt;
            ofSetHexColor(0x000000);
            ofDrawBitmapString(num, x + PORTAL_RADIUS / 2, y + PORTAL_RADIUS);
        }
    }
    
    // DRAW TRAMPOLINES
    // ------------------------
    for (int i = 0; i < myApp->trampolines.size(); i++){
        shared_ptr<ofxBox2dRect> tr = myApp->trampolines[i].ptr;
        if (tr->body->isHit()){
            ofSetHexColor(0xff0000);
        } else {
            ofSetHexColor(0xffff00);
        }
        tr->draw();
        string text = instructions.pianoNotes[tr->body->getSoundFile()];
        ofSetHexColor(0x000000);
        ofDrawBitmapString(text, tr->getPosition().x, tr->getPosition().y);
    }

    
    // DRAW LINES
    // ----------------------
    ofNoFill();
    ofSetHexColor(0x444342);
    for (int i = 0; i < myApp->lines.size(); i++){
        ofSetColor(0);
        myApp->lines[i]->draw();
    }

    if (myApp->currentMode == LINE) myApp->currentLine.draw();
    
    // DRAW STREAMS
    // ----------------------
    for (int i = 0; i < myApp->streams.size(); i++){
        ofSetColor(0);
        ofDrawCircle(myApp->streams[i].x, myApp->streams[i].y, myApp->CIRCLE_RADIUS);
    }
    
    // DRAW PORTALS
    // ----------------------------
    for (size_t i = 0; i < myApp->inPortals.size(); i++){

        ofSetColor(255);
        float x = myApp->inPortals[i].coordinates.x;
        float y = myApp->inPortals[i].coordinates.y;
        
        // this spins them
        ofPushMatrix();
        ofTranslate(x, y);
        myApp->inPortals[i].angle = myApp->inPortals[i].angle + 2;
        ofRotateZ(myApp->inPortals[i].angle);

        // this draws the pair number on the lower corner
        myApp->inPortal.draw(-PORTAL_RADIUS, -PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
        ofPopMatrix();
        int num = myApp->inPortals[i].pairNumber;
        ofSetHexColor(0x000000);
        ofDrawBitmapString(num, x + PORTAL_RADIUS / 2, y + PORTAL_RADIUS);

    }
    for (size_t i = 0; i < myApp->outPortals.size(); i++){
        ofSetColor(255);
        float x = myApp->outPortals[i].coordinates.x;
        float y = myApp->outPortals[i].coordinates.y;
        
        ofPushMatrix();
        ofTranslate(x, y);
        myApp->outPortals[i].angle = myApp->outPortals[i].angle + 2;
        ofRotateZ(myApp->outPortals[i].angle);
        myApp->outPortal.draw(-PORTAL_RADIUS, -PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
        ofPopMatrix();
        int num = myApp->outPortals[i].pairNumber;
        ofSetHexColor(0x000000);
        ofDrawBitmapString(num, x + PORTAL_RADIUS / 2, y + PORTAL_RADIUS);
    }
    
    // DRAW SELECTED RED BOX
    // ------------------------
    if (myApp->currentMode == SELECTION){
        ofSetColor(255,0,0);    // red, 50% transparent
        ofNoFill();
        switch (myApp->selected.mode){
            case RECTANGLE:{
                shared_ptr<ofxBox2dBaseShape> selectedRec = myApp->selected.ptr;
                ofVec2f pos = selectedRec.get()->getPosition();
                ofDrawRectangle(pos.x - PORTAL_RADIUS, pos.y - PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
                break;
            }
            case TRAMPOLINE:{
                shared_ptr<ofxBox2dBaseShape> selectedTramp = myApp->selected.ptr;
                ofVec2f pos = selectedTramp.get()->getPosition();
                ofDrawRectangle(pos.x - PORTAL_RADIUS, pos.y - PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
                break;
            }
            case LINE:{
                ofDrawRectangle(myApp->selected.coordinates.x - PORTAL_RADIUS, myApp->selected.coordinates.y - PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
                break;
            }
            case STREAM:
            {
                int boxWidth = myApp->CIRCLE_RADIUS * 2 + 6;
                b2Vec2 coord = myApp->selected.coordinates;
                ofDrawRectangle(coord.x - boxWidth / 2, coord.y - boxWidth / 2, boxWidth, boxWidth);
                break;
            }
            case PORTAL_IN:
            case PORTAL_OUT:
                b2Vec2 coord = myApp->selected.coordinates;
                ofDrawRectangle(coord.x - PORTAL_RADIUS, coord.y - PORTAL_RADIUS, PORTAL_RADIUS * 2, PORTAL_RADIUS * 2);
                break;
        }
        
    }
    
    // DRAW CHECKPOINT FLAGS
    ofSetColor(255);
    for (int i = 0; i < myApp->checkData.size(); i++){
        ofVec2f pos = ofVec2f(myApp->checkData[i].x, myApp->checkData[i].y);

        ofRectangle flag = ofRectangle(pos.x - (myApp->FLAG_IMAGE_SIZE / 2), pos.y + (myApp->FLAG_IMAGE_SIZE / 2), myApp->FLAG_IMAGE_SIZE, -myApp->FLAG_IMAGE_SIZE);
        myApp->flagImage.draw(flag);
    }
    
    
//    // DRAW PRECOMPUTED TRAJECTORIES
//    glColor3f(1,0,0);
////    ofDrawLine(ofPoint(0, 0), ofPoint(300, 300));
//
//    glBegin(GL_LINES);
//    vector<b2Vec2> test;
//    test.push_back(b2Vec2(400, 0));
//    test.push_back(b2Vec2(400, 10));
//    test.push_back(b2Vec2(400, 20));
//    test.push_back(b2Vec2(400, 30));
//    test.push_back(b2Vec2(400, 40));
//    test.push_back(b2Vec2(400, 50));
//    test.push_back(b2Vec2(400, 60));
//    test.push_back(b2Vec2(400, 70));
//    test.push_back(b2Vec2(400, 80));
//    test.push_back(b2Vec2(400, 90));
//    test.push_back(b2Vec2(400, 100));
//    test.push_back(b2Vec2(400, 110));
//
//    for (int i = 0; i < 10; i++){
//        glVertex2f(test[i].x, test[i].y);
//    }
//    glEnd();
//    for (int i = 0; i < myApp->streams.size(); i++){
//        b2Vec2 startingPosition = myApp->streams[i];
//        cout << "stream" << i << ":" << startingPosition.x << "," << startingPosition.y << endl;
//
//        b2Vec2 startingVelocity = b2Vec2(5,0);
//        for (int i = 0; i < 540; i+=3) { // three seconds at 60fps
//            b2Vec2 trajectoryPosition = getTrajectoryPoint( startingPosition, startingVelocity, i );
//            glVertex2f(trajectoryPosition.x, trajectoryPosition.y );
//        }
//        glEnd();
//    }
//    
    
    myApp->myCamera.end();
    
    
    // DRAW MOUSE
    int MOUSE_IMAGE_SIZE = myApp->MOUSE_IMAGE_SIZE;
    ofRectangle mouse = ofRectangle(myApp->mousePos.x - (MOUSE_IMAGE_SIZE / 2) + 2, myApp->mousePos.y, MOUSE_IMAGE_SIZE, MOUSE_IMAGE_SIZE);
    ofSetColor(255);
    switch (myApp->currentMode){
        case SELECTION:
            if (myApp->mouseIsDown){
                MOUSE_IMAGE_SIZE = 25;
                mouse = ofRectangle(myApp->mousePos.x - (MOUSE_IMAGE_SIZE / 2) + 2, myApp->mousePos.y, MOUSE_IMAGE_SIZE, MOUSE_IMAGE_SIZE);
                myApp->clenchedMouse.draw(mouse);
            } else {
                myApp->mouseHand.draw(mouse);
            }
            break;
        case PORTAL_IN:
            mouse = ofRectangle(myApp->mousePos.x - (MOUSE_IMAGE_SIZE / 2), myApp->mousePos.y - (MOUSE_IMAGE_SIZE / 2), MOUSE_IMAGE_SIZE, MOUSE_IMAGE_SIZE);
            myApp->inPortal.draw(mouse);
            break;
        case PORTAL_OUT:
            mouse = ofRectangle(myApp->mousePos.x - (MOUSE_IMAGE_SIZE / 2), myApp->mousePos.y - (MOUSE_IMAGE_SIZE / 2), MOUSE_IMAGE_SIZE, MOUSE_IMAGE_SIZE);
            myApp->outPortal.draw(mouse);
            break;
        case RECTANGLE:
            mouse = ofRectangle(myApp->mousePos.x - (MOUSE_IMAGE_SIZE / 2), myApp->mousePos.y - (MOUSE_IMAGE_SIZE / 2), MOUSE_IMAGE_SIZE, MOUSE_IMAGE_SIZE);
            ofHideCursor();
            myApp->rectangleIcon.draw(mouse);
            break;
        case TRAMPOLINE:{
            mouse = ofRectangle(myApp->mousePos.x - (MOUSE_IMAGE_SIZE / 2), myApp->mousePos.y - (MOUSE_IMAGE_SIZE / 2), MOUSE_IMAGE_SIZE, MOUSE_IMAGE_SIZE);
            ofHideCursor();
            myApp->trampolineIcon.draw(mouse);
            break;
        }
        case STREAM:{
            ofHideCursor();
            int s = MOUSE_IMAGE_SIZE - 3;
            mouse = ofRectangle(myApp->mousePos.x - (s / 2), myApp->mousePos.y - (s / 2), s, s);
            myApp->circleIcon.draw(mouse);
            break;
        }
        default:
            break;
    }

    // DISPLAY TIMER
    // ----------------
    string timerText = ofToString(myApp->timer.GetMilliseconds());
    ofSetHexColor(0x000000);
    //    ofDrawBitmapString(timerText, 500, 24);
}

b2Vec2 drawFunction::getTrajectoryPoint( b2Vec2& startingPosition, b2Vec2& startingVelocity, float n )
{
    //velocity and gravity are given per second but we want time step values here
    float t = 1 / 60.0f; // seconds per time step (at 60fps)
    b2Vec2 stepVelocity = t * startingVelocity; // m/s
    
    b2Vec2 stepGravity = t * t * myApp->box2d.getWorld()->GetGravity(); // m/s/s
    
    return startingPosition + n * stepVelocity + 0.5f * (n*n+n) * stepGravity;
}