//
//  instructions.cpp
//  collide
//
//  Created by Andrew Lee on 12/9/15.
//
//

#include "instructions.h"

string beginnerNotes[] = {
    "[1] - Set tone to C",
    "[2] - Set tone to C#",
    "[3] - Set tone to D",
    "[4] - Set tone to D#",
    "[5] - Set tone to E",
    "[6] - Set tone to F",
    "[7] - Set tone to F#",
    "[8] - Set tone to G",
    "[9] - Set tone to G#",
    "[0] - Set tone to A",
    "[-] - Set tone to A#",
    "[=] - Set tone to B"
};

string beginner2[] = {
    "[Fn + Delete] - Clear everything",
    "[Fn + z] - Undo (in selected category)",
    "[s] - Save world",
    "[l] - Load world",
    "[m] - More..."
};

string advancedModes[] = {
    "[q] - line",
    "[w] - starting point",
    "[r] - rectangle",
    "[R] - rotating rectangle",
    "[t] - trampoline",
    "[T] - rotating trampoline",
    "[fn] - hold for exact angles",
    "[y] - entry-portal",
    "[u] - exit-portal",
    "[v] - edit/scroll"
};

string advancedNotes[] = {
    "[shift] - raise half step",
    "[command] - lower half step",
    "[z] - raise one octave",
    "[x] - lower one octave"
};

string advancedSelection[] = {
    "[delete] - remove object",
    "[a-g] - pitch notes",
    "[1-9] - portal numbers"
};

string advancedUseful[] = {
    "[j] - set checkpoint",
    "[k] - delete checkpoint",
    "[n] - show ball trajectories",
//    "[tab] - continuous stream",
    "[`] - clear balls",
    "[i] - zoom in",
    "[o] - zoom out",
    "[h] - hide instructions",
    "[m] - Less..."
};

instructions::instructions(testApp* ptr){
    myApp = ptr;
}


void instructions::drawInstructions(){
    if (myApp->instructionMode == ADVANCED){
        string note = "CURRENT NOTE:";
        ofDrawBitmapStringHighlight(note, ofPoint(970, 30));
        string currentNote = pianoNotes[myApp->currentSound];
        ofDrawBitmapString(currentNote, 1080, 30);
        
        string check = "CHECKPOINT:";
        ofDrawBitmapStringHighlight(check, ofPoint(1155, 30));
        string on = "On";
        string off = "Off";
        if (myApp->checkPoint){
            ofDrawBitmapString(on, ofPoint(1250, 30));
        } else {
            ofDrawBitmapString(off, ofPoint(1250, 30));
        }
    }
    
    if (!myApp->showInstructions){
        for (int i = 0; i < myApp->NUM_BUTTONS; i++){
            myApp->buttons[i].hide();
        }
        return;
    }
    switch (myApp->instructionMode){
        case INSTRUCTIONAL:
            break;
        case BEGINNER:{
            for (int i = 0; i < myApp->NUM_BUTTONS; i++){
                myApp->buttons[i].show();
            }
            // DISPLAY TEXT
            // ----------------------
            // "Play" instruction
            string info = "1) Press the PLAY button or 'p' to start your sequence:";
            ofSetHexColor(0x444342);
            ofDrawBitmapStringHighlight(info, ofPoint(30,30));
            
            string click = "2) CLICK and DRAG to add objects to change course!";
            ofDrawBitmapStringHighlight(click, ofPoint(30, 60));
            
            string rectangles = " - bounce the ball off these to play notes";
            ofDrawBitmapString(rectangles, ofPoint(100, 90));
            string lines = " - use lines when you don't want sound";
            ofDrawBitmapString(lines, ofPoint(100, 140));
            
            // Pitch instructions
            string pitch = "3) Platforms can have different notes:";
            ofDrawBitmapStringHighlight(pitch, ofPoint(30, 180));
            int pointY;
            for (int i = 0; i < 12; i++){
                string text = beginnerNotes[i];
                pointY = i * myApp->TEXT_HEIGHT + 200;
                int currentSound = (myApp->currentSound - 16) % 12;
                if (i == currentSound){
                    ofDrawBitmapStringHighlight(text, ofPoint(30, pointY), ofColor::yellow, ofColor::black);
                } else {
                    ofDrawBitmapString(text, 30, pointY);
                }
            }
            
            // Other instructions
            string fun = "4) Most importantly... HAVE FUN!";
            // s for save, l for load, b for advanced keyboard
            ofDrawBitmapStringHighlight(fun, ofPoint(30, 435));
            for (int i = 0; i < 5; i++){
                string text = beginner2[i];
                pointY = i * myApp->TEXT_HEIGHT + 455;
                ofDrawBitmapString(text, 30, pointY);
            }
            break;
        }
            
        case ADVANCED:{
            for (int i = 0; i < myApp->NUM_BUTTONS; i++){
                myApp->buttons[i].hide();
            }
            
            // DISPLAY TEXT
            // ----------------------
            // Mode instructions
            string mode = "ADVANCED MODES:";
            ofDrawBitmapStringHighlight(mode, ofPoint(30, 30));
            
            for (int i = 0; i < 10; i++){
                string text = advancedModes[i];
                int pointY = i * myApp->TEXT_HEIGHT + 50;
                ofDrawBitmapString(text, 30, pointY);
            }
            
            // Selection instructions
            string selection = "SELECTION: ('v')";
            ofDrawBitmapStringHighlight(selection, ofPoint(30, 250));
            
            for (int i = 0; i < 3; i++){
                string text = advancedSelection[i];
                int pointY = i * myApp->TEXT_HEIGHT + 270;
                ofDrawBitmapString(text, 30, pointY);
            }

            
            // Pitch instructions
            string pitch = "PITCH:";
            ofDrawBitmapStringHighlight(pitch, ofPoint(30, 340));
        
            for (int i = 0; i < 4; i++){
                string text = advancedNotes[i];
                int pointY = i * myApp->TEXT_HEIGHT + 360;
                ofDrawBitmapString(text, 30, pointY);
            }

            // Useful instructions
            string useful = "USEFUL FUNCTIONS:";
            ofDrawBitmapStringHighlight(useful, ofPoint(30, 450));
            
            for (int i = 0; i < 8; i++){
                string text = advancedUseful[i];
                int pointY = i * myApp->TEXT_HEIGHT + 470;
                ofDrawBitmapString(text, 30, pointY);
            }
            break;
        
        }
    }
    

    
    
    
}