//
//  loadingWorld.cpp
//  collide
//
//  Created by Andrew Lee on 11/17/15.
//
//

#include "loadingWorld.hpp"
#include "ofxFileDialog.h"


loadingWorld::loadingWorld(testApp* ptr)
{
    myApp = ptr;
}

void loadingWorld::loadWorld(string file){
    vector <platform> newRectangles;
    vector <shared_ptr<ofxBox2dEdge> > newLines;
    vector <platform> newTrampolines;
    vector<b2Vec2> newStreams;
    
    vector <string>  types = ofSplitString(ofBufferFromFile(file).getText(), "#");
    string streamData = types[0];
    string lineData = types[1];
    string rectangleData = types[2];
    string trampolineData = types[3];
    string inPortalsData = types[4];
    string outPortalsData = types[5];
    
    // Loading in stream data:
    // [100, 125, 344, 344, 50, 123]
    // becomes 3 streams
    // stream1{100, 125}, stream2{344, 344}, and stream3{50, 123}
    vector <string>  ptsStr = ofSplitString(streamData, ",");
    for (int i = ptsStr.size() - 3; i >= 0; i-=2){
        float x = ofToFloat(ptsStr[i]);
        float y = ofToFloat(ptsStr[i+1]);
        newStreams.push_back(b2Vec2(x, y) );
    }
    
    // Loading in line data:
    // [30, 45, 12, 15, 70, 70 / 24, 24, 100, 100 / 1, 4, 6, 8, 2, 2]
    // becomes 3 lines
    // line1({30, 45}, {12, 15}, {70, 70}), line2({24, 24}, {100, 100}), line3({1, 4}, {6, 8}, {2, 2})
    vector<string> individualLines = ofSplitString(lineData, "/");
    for (int l = individualLines.size() - 1; l >= 0; l--){
        if (individualLines[l] != ""){
            vector<string> lineData = ofSplitString(individualLines[l], ",");
            vector <ofVec2f> linePoints;
            
            for (int i = 0; i < lineData.size() - 1; i+=2){
                int x = ofToInt(lineData[i]);
                int y = ofToInt(lineData[i+1]);
                linePoints.push_back(ofVec2f(x, y));
            }
            shared_ptr <ofxBox2dEdge> line = shared_ptr <ofxBox2dEdge>(new ofxBox2dEdge);
            line.get()->addVertexes(linePoints);
            line.get()->setPhysics(0.0, 0.5, 0.5);
            line->create(myApp->box2d.getWorld());
            newLines.push_back(line);
        }
        
    }
    
    // Loading in rectangle data:
    // [10, 10, 2, 2, 125, 0, 15, 30, 8, 8, 30, 1]
    // becomes 2 rectangles (x, y, width, height) with rotation angle and soundFile
    // rect1(10, 10, 2, 2) rotated 125 degrees w/ sound file 0, rect2(15, 30, 8, 8) rotated 30 degrees with sound 1
    
    // appended to it is platform data (id, rotating, startingAngle, timesBeforeExpire)
    // rotating is a bool, 1 is true and 0 is false
    
    vector <string>  recPoints = ofSplitString(rectangleData, ",");
    for (int i = recPoints.size() - 11; i >= 0; i-=10){
        
        
        float x = ofToFloat(recPoints[i]);
        float y = ofToFloat(recPoints[i+1]);
        float w = ofToFloat(recPoints[i+2]);
        float h = ofToFloat(recPoints[i+3]);
        float angle = ofToFloat(recPoints[i+4]);
        int soundFile = ofToInt(recPoints[i+5]);
        
        //Platform Data
        int id = ofToInt(recPoints[i+6]);
        int rotating = ofToInt(recPoints[i+7]);
        float startingAngle = ofToFloat(recPoints[i+8]);
        int timesBeforeExpire = ofToInt(recPoints[i+9]);
        
        ofPtr<ofxBox2dRect> rect = ofPtr<ofxBox2dRect>(new ofxBox2dRect);
        rect.get()->setPhysics(0,0.2,0.2);
        rect.get()->setup(myApp->box2d.getWorld(), x, y, w, h);
        rect.get()->body->SetType(b2_staticBody);
        b2Body* rectBody = rect.get()->body;
        
        rectBody->SetTransform(rectBody->GetPosition(), angle * 3.14159 / 180);
        
        rectBody->setSoundFile(soundFile);
        
        platform p;
        p.ptr = rect;
        p.id = id;
        if (rotating == 1){
            p.rotating = true;
        } else {
            p.rotating = false;
        }
        p.startingAngle = startingAngle;
        p.timesBeforeExpire = timesBeforeExpire;
        newRectangles.push_back(p);
    }
    
    // Loading in trampoline data:
    // [10, 10, 2, 2, 125, 0, 10, 15, 30, 8, 8, 30, 1, 10]
    // becomes 2 trampolines (x, y, width, height) with rotation angle, soundFile, and acceleration
    // tramp1(10, 10, 2, 2) rotated 125 degrees w/ sound file 0, tramp2(15, 30, 8, 8) rotated 30 degrees with sound 1, both with acceleration of 10
    vector <string>  trampPoints = ofSplitString(trampolineData, ",");
    for (int i = trampPoints.size() - 12; i >= 0; i-=11){
        
        float x = ofToFloat(trampPoints[i]);
        float y = ofToFloat(trampPoints[i+1]);
        float w = ofToFloat(trampPoints[i+2]);
        float h = ofToFloat(trampPoints[i+3]);
        float angle = ofToFloat(trampPoints[i+4]);
        int soundFile = ofToInt(trampPoints[i+5]);
        int accel = ofToInt(trampPoints[i+6]);
        
        // Platform Data
        int id = ofToInt(trampPoints[i+7]);
        int rotating = ofToInt(trampPoints[i+8]);
        float startingAngle = ofToFloat(trampPoints[i+9]);
        int timesBeforeExpire = ofToInt(trampPoints[i+10]);
        
        ofPtr<ofxBox2dRect> tramp = ofPtr<ofxBox2dRect>(new ofxBox2dRect);
        tramp.get()->setPhysics(0,0.2,0.2);
        tramp.get()->setup(myApp->box2d.getWorld(), x, y, w, h);
        tramp.get()->body->SetType(b2_staticBody);
        b2Body* trampBody = tramp.get()->body;
        trampBody->SetTransform(trampBody->GetPosition(), angle * 3.14159 / 180);
        trampBody->setSoundFile(soundFile);
        trampBody->setAcceleration(accel);
        
        platform p;
        p.ptr = tramp;
        p.id = id;
        if (rotating == 1){
            p.rotating = true;
        } else {
            p.rotating = false;
        }
        p.startingAngle = startingAngle;
        p.timesBeforeExpire = timesBeforeExpire;
        newTrampolines.push_back(p);
    }
    
    // Loading in in-portal data
    // [58, 35, 2]
    // First coordinate is x-position
    // Second coordinate is y-position
    // Third coordinate is pairNumber
    vector<portal> newInPortals;
    vector <string>  inPortalsStr = ofSplitString(inPortalsData, ",");
    for (int i = inPortalsStr.size() - 4; i >= 0; i-=3){
        float x = ofToFloat(inPortalsStr[i]);
        float y = ofToFloat(inPortalsStr[i+1]);
        int pairNumber = ofToInt(inPortalsStr[i+2]);
        portal p;
        p.coordinates = b2Vec2(x, y);
        p.pairNumber = pairNumber;
        newInPortals.push_back(p);
    }
    
    vector<portal> newOutPortals;
    vector <string>  outPortalsStr = ofSplitString(outPortalsData, ",");
    for (int i = outPortalsStr.size() - 4; i >= 0; i-=3){
        float x = ofToFloat(outPortalsStr[i]);
        float y = ofToFloat(outPortalsStr[i+1]);
        int pairNumber = ofToInt(outPortalsStr[i+2]);
        portal p;
        p.coordinates = b2Vec2(x, y);
        p.pairNumber = pairNumber;
        newOutPortals.push_back(p);
    }

    
    std::swap( myApp->rectangles, newRectangles );
    std::swap( myApp->lines, newLines );
    std::swap( myApp->trampolines, newTrampolines );
    std::swap( myApp->streams, newStreams );
    std::swap( myApp->inPortals, newInPortals);
    std::swap( myApp->outPortals, newOutPortals);
    
    int greatestID = 0;
    for (platform r : myApp->rectangles){
        if (r.id > greatestID) greatestID = r.id;
    }
    for (platform t : myApp->trampolines){
        if (t.id > greatestID) greatestID = t.id;
    }
    myApp->idCounter = greatestID + 1;
}

void loadingWorld::saveWorld(){
    string defaultName = "MyWorld.dat";
    string message = "Save your world!";
    string defaultPath = ofToDataPath("", true);
    ofFileDialogResult result = customSaveDialog(defaultName, message, defaultPath);
    
    ofBuffer dataBuffer;
    
    // Write stream data
    for (int i = 0; i < myApp->streams.size(); i++){
        b2Vec2 point = myApp->streams[i];
        string s = ofToString(point.x) + "," + ofToString(point.y) + ",";
        dataBuffer.append(s);
    }
    
    // Write line data
    dataBuffer.append("#");
    for (int i = 0; i < myApp->lines.size(); i++){
        shared_ptr<ofxBox2dEdge>  edge = myApp->lines[i];
        vector<ofPoint> points = edge->getVertices();
        for (int i = 0; i < points.size(); i++){
            string s = ofToString(points[i].x) + "," + ofToString(points[i].y) + ",";
            dataBuffer.append(s);
        }
        dataBuffer.append("/");
    }

    // Write rectangle data
    dataBuffer.append("#");
    for (int i = 0; i < myApp->rectangles.size(); i++){
        shared_ptr<ofxBox2dRect> rect = myApp->rectangles[i].ptr;
        ofVec2f position = rect->getPosition();
        int sound = rect->body->getSoundFile();
        float angle = rect->getRotation();
        float x = position.x;
        float y = position.y;
        float w = rect->getWidth();
        float h = rect->getHeight();
        
        platform p = myApp->rectangles[i];
        int id = p.id;
        int rotating;
        if (p.rotating){
            rotating = 1;
        } else {
            rotating = 0;
        }
        float startingAngle = p.startingAngle;
        int timesBeforeExpire = p.timesBeforeExpire;
        
        
        string s = ofToString(x) + "," + ofToString(y) + "," + ofToString(w) + "," + ofToString(h) + "," + ofToString(angle) + "," + ofToString(sound) + ",";
        string platformData = ofToString(id) + "," + ofToString(rotating) + "," + ofToString(startingAngle) + "," + ofToString(timesBeforeExpire) + ",";
        s = s + platformData;
        dataBuffer.append(s);
    }
    
    // Write trampoline data
    dataBuffer.append("#");
    for (int i = 0; i < myApp->trampolines.size(); i++){
        shared_ptr<ofxBox2dRect> tramp = myApp->trampolines[i].ptr;
        ofVec2f position = tramp->getPosition();
        int accel = tramp->body->getAcceleration();
        int sound = tramp->body->getSoundFile();
        float angle = tramp->getRotation();
        float x = position.x;
        float y = position.y;
        float w = tramp->getWidth();
        float h = tramp->getHeight();
        
        
        platform p = myApp->trampolines[i];
        int id = p.id;
        int rotating;
        if (p.rotating){
            rotating = 1;
        } else {
            rotating = 0;
        }
        float startingAngle = p.startingAngle;
        int timesBeforeExpire = p.timesBeforeExpire;
        
        string s = ofToString(x) + "," + ofToString(y) + "," + ofToString(w) + "," + ofToString(h) + "," + ofToString(angle) + "," + ofToString(sound) + "," + ofToString(accel) + ",";
        string platformData = ofToString(id) + "," + ofToString(rotating) + "," + ofToString(startingAngle) + "," + ofToString(timesBeforeExpire) + ",";
        s = s + platformData;
        dataBuffer.append(s);
    }
    
    // Write in-portal data
    dataBuffer.append("#");
    for (int i = 0; i < myApp->inPortals.size(); i++){
        portal p = myApp->inPortals[i];
        float x = p.coordinates.x;
        float y = p.coordinates.y;
        int pairNumber = p.pairNumber;
        string s = ofToString(x) + "," + ofToString(y) + "," + ofToString(pairNumber) + ",";
        dataBuffer.append(s);
    }
    
    // Write out-portal data
    dataBuffer.append("#");
    for (int i = 0; i < myApp->outPortals.size(); i++){
        portal p = myApp->outPortals[i];
        float x = p.coordinates.x;
        float y = p.coordinates.y;
        int pairNumber = p.pairNumber;
        string s = ofToString(x) + "," + ofToString(y) + "," + ofToString(pairNumber) + ",";
        dataBuffer.append(s);
    }
    
    ofBufferToFile(result.getPath(), dataBuffer);
}

