//
//  selection.cpp
//  collide
//
//  Created by Andrew Lee on 11/21/15.
//
//

#include "selection.hpp"

selection::selection(testApp* ptr){
    myApp = ptr;
}

bool selection::withinLine(vector<ofPoint> vertices, ofPoint mouse){
    for (int i = 0; i < vertices.size(); i++){
        b2Vec2 circleCenter = b2Vec2(vertices[i].x, vertices[i].y);
        if (withinCircle(circleCenter, mouse, 5)){
            return true;
        }
    }
    return false;
}

bool selection::withinCircle(b2Vec2 circleCenter, ofVec2f point, float radius){
    return (pow(circleCenter.x - point.x, 2) + pow(circleCenter.y - point.y, 2) <= pow(radius, 2));
}

bool selection::withinFixtures(b2Fixture* head, b2Vec2 point){
    while (true){
        if (head == NULL) return false;
        if (head->TestPoint(point)) return true;
        head = head->GetNext();
    }
}

bool selection::pnpoly(int nvert, float *vertx, float *verty, float testx, float testy)
{
    int i, j, c = 0;
    for (i = 0, j = nvert-1; i < nvert; j = i++) {
        if ( ((verty[i]>testy) != (verty[j]>testy)) &&
            (testx < (vertx[j]-vertx[i]) * (testy-verty[i]) / (verty[j]-verty[i]) + vertx[i]) )
            c = !c;
    }
    return c;
}

bool selection::detectSelection(float x, float y){
    ofVec2f mouseClick = ofVec2f(x,y);
    b2Vec2 boxPoint = b2Vec2(x / OFX_BOX2D_SCALE, y / OFX_BOX2D_SCALE);
    ofPoint mousePoint = ofPoint(x, y);

    // Lines
    for (int i = 0; i < myApp->lines.size(); i++){
        ofPtr<ofxBox2dEdge> linePtr = myApp->lines[i];
        
        // Iterate through each pair of vertices...
        ofPoint p1, p2;
        int N = linePtr.get()->size();
        p1 = (*linePtr.get())[0];
        for (int i = 1; i<N; i++) {
            p2 = (*linePtr.get())[i % N];
            
            // ALGEBRAIC WAY
            float xDiff;
            float yDiff;
            float deltaY = (p2.y - p1.y);
            if (deltaY == 0){
                xDiff = 0;
                yDiff = myApp->RECTANGLE_WIDTH / 2;
            } else {
                float deltaX = (p2.x - p1.x);
                float m_prime = -(deltaX / deltaY);
                xDiff = myApp->RECTANGLE_WIDTH / (2 * sqrt(pow(m_prime, 2) + 1));
                yDiff = xDiff * m_prime;
            }

            // Bounding rectangles for each pair of vertices in line object
            ofVec2f P1(p1.x - xDiff, p1.y - yDiff);
            ofVec2f P2(p1.x + xDiff, p1.y + yDiff);
            ofVec2f P3(p2.x + xDiff, p2.y + yDiff);
            ofVec2f P4(p2.x - xDiff, p2.y - yDiff);

            // Test boundaries of each rectangle
            int nvert = 4;
            float vertx[] = {P1.x, P2.x, P3.x, P4.x};
            float verty[] = {P1.y, P2.y, P3.y, P4.y};
            float testx = x;
            float testy = y;
            if (pnpoly(nvert, vertx, verty, testx, testy)){
                myApp->selected.mode = LINE;
                myApp->selected.ptr = linePtr;
                ofPoint closestPoint = linePtr.get()->getPointAtPercent(.5);
                myApp->selected.coordinates = b2Vec2(closestPoint.x, closestPoint.y);
                return true;
            }
            
            p1 = p2;
        }
    }
    
    // Rectangles
    for (int i = 0; i < myApp->rectangles.size(); i++){
        ofPtr<ofxBox2dRect> recPtr = myApp->rectangles[i].ptr;
        b2Fixture* fixtureList = recPtr.get()->body->GetFixtureList();
        if (withinFixtures(fixtureList, boxPoint)){
            myApp->selected.mode = RECTANGLE;
            myApp->selected.ptr = myApp->rectangles[i].ptr;
            myApp->selected.coordinates = b2Vec2(0, 0);
            myApp->selected.id = myApp->rectangles[i].id;
            return true;
        }
    }

    // Trampolines
    for (int i = 0; i < myApp->trampolines.size(); i++){
        ofPtr<ofxBox2dRect> trampPtr = myApp->trampolines[i].ptr;
        b2Fixture* fixtureList = trampPtr.get()->body->GetFixtureList();
        if (withinFixtures(fixtureList, boxPoint)){
            myApp->selected.mode = TRAMPOLINE;
            myApp->selected.ptr = myApp->trampolines[i].ptr;
            myApp->selected.coordinates = b2Vec2(0, 0);
            myApp->selected.id = myApp->trampolines[i].id;
            return true;
        }
    }
    
    // streams
    for (int i = 0; i < myApp->streams.size(); i++){
        if (withinCircle(myApp->streams[i], mouseClick, myApp->CIRCLE_RADIUS)){
            myApp->selected.mode = STREAM;
            myApp->selected.ptr = NULL;
            myApp->selected.coordinates = myApp->streams[i];
            myApp->selected.intermittent = myApp->streams[i];
            return true;
        }
    }
    
    //inPortals
    for (int i = 0; i < myApp->inPortals.size(); i++){
        if (withinCircle(myApp->inPortals[i].coordinates, mouseClick, myApp->PORTAL_RADIUS)){
            myApp->selected.mode = PORTAL_IN;
            myApp->selected.ptr = NULL;
            myApp->selected.coordinates = myApp->inPortals[i].coordinates;
            myApp->selected.intermittent = myApp->inPortals[i].coordinates;
            myApp->selected.metaInt = myApp->inPortals[i].pairNumber;
            return true;
        }
    }
    
    //outPortals
    for (int i = 0; i < myApp->outPortals.size(); i++){
        if (withinCircle(myApp->outPortals[i].coordinates, mouseClick, myApp->PORTAL_RADIUS)){
            myApp->selected.mode = PORTAL_OUT;
            myApp->selected.ptr = NULL;
            myApp->selected.coordinates = myApp->outPortals[i].coordinates;
            myApp->selected.intermittent = myApp->outPortals[i].coordinates;
            myApp->selected.metaInt = myApp->outPortals[i].pairNumber;
            return true;
        }
    }
    
    // NOTHING was clicked on
    myApp->selected.mode = SELECTION;
    myApp->selected.ptr = NULL;
    myApp->selected.coordinates = b2Vec2(0, 0);
    return false;
    
}