#pragma once
#include "ofMain.h"
#include "ofxBox2d.h"
#include <iostream>
#include <ctime>
#include "ofxSimpleButton.h"
#include "b2dJson.h"

#define OCTAVE 12

class SoundData {
public:
	int	 soundID;
	bool bHit;
};



enum Mode { STREAM, LINE, RECTANGLE, TRAMPOLINE, SELECTION, PORTAL_IN, PORTAL_OUT };
enum InstructionMode { INSTRUCTIONAL, BEGINNER, ADVANCED };

typedef struct {
    b2Vec2 coordinates;
    int pairNumber;
    float angle;
} portal;

typedef struct {
    shared_ptr<ofxBox2dRect> ptr;
    int id;
    bool rotating;
    float startingAngle;
    int timesBeforeExpire;
} platform;

// -------------------------------------------------
class testApp : public ofBaseApp {
	
public:
    
    // DEFAULT
    // --------------------------
	void setup();
	void update();
	void draw();
	void keyPressed(int key);
	void keyReleased(int key);
	void mouseMoved(int x, int y);
	void mouseDragged(int x, int y, int button);
	void mousePressed(int x, int y, int button);
	void mouseReleased(int x, int y, int button);
	void resized(int w, int h);
    
    // 2D DRAWING WORLD
    // -------------------------
	void contactStart(ofxBox2dContactArgs &e);
	void contactEnd(ofxBox2dContactArgs &e);
    time_t initialTime;
    ofxBox2d box2d;
    
    // ADDED FUNCTIONS
    // --------------------------
    void loadWorld(string file);
    void saveWorld();
    bool intersection(b2Vec2 portalCenter, ofVec2f circleCenter);
    bool checkTeleport(int circleIndex);
    bool detectSelection(int x, int y);
    void deleteOffScreen();


//PRIVATE but switching to public for now
public:
    
    // CONSTANTS:
    // -------------------------
    const static int NUM_BUTTONS = 5;
    const int SCREEN_WIDTH = 1280;
    const int SCREEN_HEIGHT = 720;
    const int CIRCLE_RADIUS = 10;
    const int PORTAL_RADIUS = 20;
    const int RECTANGLE_WIDTH = 10;
    const int TRICKLE_FREQUENCY = 150; // the higher the less frequent
    const int TRAMPOLINE_ACCEL = 15;
    const int TEXT_HEIGHT = 18;
    const int MIN_RECT_LENGTH = 15;
    const int MIN_LINE_LENGTH = 15;
    const int MOUSE_IMAGE_SIZE = 20;
    const int FLAG_IMAGE_SIZE = 25;

    
    // CIRCLES/LINES
    // ------------------------
	vector <shared_ptr<ofxBox2dCircle> > circles;
    vector <shared_ptr<ofxBox2dEdge> > lines;
    ofPolyline currentLine;
    
    // RECTANGLES/TRAMPOLINES
    // -------------------------
    b2Vec2 recPoint1;
    b2Vec2 recPoint2;
    typedef struct {
        float x;
        float y;
        float w;
        float h;
        float angle;
    } myRectangle;
    myRectangle currentRectangle;
    vector<platform> rectangles;

//    vector<shared_ptr<ofxBox2dRect> > rectangles;
    vector<platform> trampolines;
    
    
    // STREAMS/PORTALS
    // ------------------------
    ofImage inPortal;
    ofImage outPortal;
    vector <b2Vec2> streams;
    vector<portal> inPortals;
    vector<portal> outPortals;

    // CAMERA/SOUNDS/OTHER
    // --------------------------
    ofCamera myCamera;
    ofSoundPlayer sound[64];
//    ofVideoPlayer video;
//    ofSoundPlayer  sound[OCTAVE * 4]; // when the ball hits we play this sound
    b2Timer timer;
    Mode currentMode = RECTANGLE;
    InstructionMode instructionMode = BEGINNER;
    bool showInstructions = true;
    int trickleCounter = 0;
    int currentSound;
    int START_NOTE = 28;
    int idCounter = 0;
    int circleIDCounter = 0;
    bool trickle = false;
    bool showBallTrajectories;
    
    // MOUSE-involved bools
    bool currentlyDrawingRectangle = false;
    bool currentlyDrawingLine = false;
    bool paused = false;
    ofVec2f mousePos;
    bool rotatingRectangles = false;
    bool shiftIsDown = false;
    bool commandIsDown = false;
    bool fnIsDown = false;
    bool mouseIsDown = false;
    bool justPressedButton = false;

    
    // CURRENT SELECTION
    typedef struct {
        shared_ptr<ofxBox2dBaseShape> ptr; //ptr to the Box2D object
        Mode mode; // what object is this?
        b2Vec2 coordinates; // where is it located?
        b2Vec2 intermittent; // for streams/portals while being dragged
        int metaInt;
        int id;
        bool dragged;
        ofVec2f difference;
    } currentSelection;
    
    currentSelection selected;
    
//    typedef struct{
//        bool dragged;
//        ofVec2f difference;
//    } draggingSelectedRectangle;
//    draggingSelectedRectangle selectedPlatform;
    
    
    // CHECKPOINT / TRAJECTORIES
    // ------------------------
    bool checkPoint = false;
    vector <shared_ptr<ofxBox2dCircle> > checkPointCircles;
    vector < vector <b2Vec2> > ballTrajectories;
    
    b2dJson json;
    string jsonData;
    
    vector<ofVec4f> checkData; // x, y, x-velocity, y-velocity
//    vector< shared_ptr < b2Body > > checkpointData;
    vector<b2BodyDef > checkpointData;
    vector<float> checkData2;
    
    // for rotating platform angles
    vector<float> checkPointRec;
    vector<float> checkPointTramp;

    ofImage flagImage;
    
    // BUTTONS/BUTTON FUNCTIONS
    // -----------------------
    ofxSimpleButton buttons[NUM_BUTTONS];
    ofRectangle playButton;
    ofImage buttonImage;

    void playButtonFunction();
    void pauseButtonFunction();
    void stopButtonFunction();
    void recButtonFunction();
    void lineButtonFunction();
    void circleButtonFunction();
    
    ofImage mouseHand;
    ofImage clenchedMouse;
    ofImage buttonOutline;
    ofImage rectangleIcon;
    ofImage trampolineIcon;
    ofImage lineIcon;
    ofImage circleIcon;
    
    
    vector<ofVec2f> testRecs;
};

